;; Generated package description from timeout.el  -*- no-byte-compile: t -*-
(define-package "timeout" "2.0.0.20250901.120627" "Throttle or debounce Elisp functions" '((emacs "24.4")) :commit "9ad5167024f96bea45669acd862615ca379feddf" :authors '(("Karthik Chikmagalur" . "karthikchikmagalur@gmail.com")) :maintainer '("Karthik Chikmagalur" . "karthikchikmagalur@gmail.com") :keywords '("convenience" "extensions") :url "https://github.com/karthink/timeout")
